# All detected features are included in "KERNEL_FEATURE_DETECTION"

# KERNEL_HAS_IBLKSIZE Detection [START]
#
# Find out whether the kernel supports inode->i_blksize.
# Note: i_blksize was removed in newer 2.6 kernels, but some distros have
#       patches that add it again.
# Note: Test just greps for i_blksize (so it might result in false positives,
#       but that is unlikely).
KERNEL_HAS_IBLKSIZE = $(shell \
	if grep i_blksize ${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 2>&1 1>/dev/null ; \
	then echo "-DKERNEL_HAS_IBLKSIZE" ; \
	fi)
#
# KERNEL_HAS_IBLKSIZE Detection [END]

# KERNEL_HAS_FLUSH_OWNER Detection [START]
#
# Find out whether the kernel vfs flush has fl_owner_t parameter.
# Note: fl_owner_t parameter was added in vanilla 2.6.18, but SLES 10 adds it
#       to its 2.6.16 kernels.
KERNEL_HAS_FLUSH_OWNER = $(shell \
	if grep "int (\*flush) (struct file \*, fl_owner_t id);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 2>&1 1>/dev/null ; \
	then echo "-DKERNEL_HAS_FLUSH_OWNER" ; \
	fi)
#
# KERNEL_HAS_FLUSH_OWNER Detection [END]

# KERNEL_HAS_STATFS_DENTRY Detection [START]
#
# Find out whether the kernel vfs statfs has a dentry parameter.
# Note: dentry parameter was added in vanilla 2.6.18, but SLES 10 adds it
#       to its 2.6.16 kernels.
KERNEL_HAS_STATFS_DENTRY = $(shell \
	if grep "int (\*statfs) (struct dentry \*, struct kstatfs \*);" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 2>&1 1>/dev/null ; \
	then echo "-DKERNEL_HAS_STATFS_DENTRY" ; \
	fi)
#
# KERNEL_HAS_STATFS_DENTRY Detection [END]

# KERNEL_HAS_DROP_NLINK Detection [START]
#
# Find out whether the kernel has a drop_nlink function.
# Note: Was added in vanilla 2.6.19, but RedHat adds it to 2.6.18.
KERNEL_HAS_DROP_NLINK = $(shell \
	if grep "drop_nlink(struct inode \*inode)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 2>&1 1>/dev/null ; \
	then echo "-DKERNEL_HAS_DROP_NLINK" ; \
	fi)
#
# KERNEL_HAS_DROP_NLINK Detection [END]

# KERNEL_HAS_CLEAR_NLINK Detection [START]
#
# Find out whether the kernel has a clear_nlink function.
# Note: Was added in vanilla 2.6.19, but RedHat adds it to 2.6.18.
KERNEL_HAS_CLEAR_NLINK = $(shell \
	if grep "clear_nlink(struct inode \*inode)" \
		${KSRCDIR_PRUNED_HEAD}/include/linux/fs.h 2>&1 1>/dev/null ; \
	then echo "-DKERNEL_HAS_CLEAR_NLINK" ; \
	fi)
#
# KERNEL_HAS_CLEAR_NLINK Detection [END]


# Combine results
KERNEL_FEATURE_DETECTION := $(KERNEL_HAS_IBLKSIZE) $(KERNEL_HAS_FLUSH_OWNER) \
	$(KERNEL_HAS_STATFS_DENTRY) $(KERNEL_HAS_DROP_NLINK) \
	$(KERNEL_HAS_CLEAR_NLINK)
