#ifndef LOGGER_H_
#define LOGGER_H_


// Note: There are two alternative implementations for the Logger class.

#define FHGFS_DIRECTLOGGER    1 /* declared in DirectLogger.h (deleted!) */
#define FHGFS_HELPERDLOGGER   2 /* declared in HelperdLogger.h */

#define FHGFS_LOGGER_TYPE     FHGFS_HELPERDLOGGER


enum LogLevel
{
   Log_ERR=0, /* system error */
   Log_CRITICAL=1, /* something the users should definitely know about */
   Log_WARNING=2, /* things that indicate or are related to a problem */
   Log_NOTICE=3, /* things that could help finding problems */
   Log_DEBUG=4, /* things that are only useful during debugging, typically with LOG_DEBUG */
   Log_SPAM=5 /* things that are typically too detailed even during normal debugging,
      definitely with LOG_DEBUG only */
};



#include "HelperdLogger.h"


#ifdef LOG_DEBUG_MESSAGES

   #define LOG_DEBUG(logger, level, contextStr, msgStr) \
      do { Logger_log(logger, level, contextStr, msgStr); } while(0)
   
   #define LOG_DEBUG_FORMATTED(logger, level, contextStr, msgStr, ...) \
      do { Logger_logFormatted(logger, level, contextStr, msgStr, ## __VA_ARGS__); } while(0)

#else

   #define LOG_DEBUG(logger, level, contextStr, msgStr)
   
   #define LOG_DEBUG_FORMATTED(logger, level, contextStr, msgStr, ...)

#endif // LOG_DEBUG_MESSAGES


#endif /*LOGGER_H_*/
