#ifndef POOLEDSOCKET_H_
#define POOLEDSOCKET_H_

#include <include_closed/common/net/sock/Socket.h>


/*
 * This is an abstract class.
 */


struct PooledSocket;
typedef struct PooledSocket PooledSocket;


static inline void _PooledSocket_init(PooledSocket* this, NicAddrType_t nicType);
static inline PooledSocket* _PooledSocket_construct(NicAddrType_t nicType);
static inline void _PooledSocket_uninit(Socket* this);
static inline void _PooledSocket_destruct(Socket* this);

// inliners
static inline fhgfs_bool PooledSocket_testAndDecExpirationCounter(PooledSocket* this);

// getters & setters
static inline fhgfs_bool PooledSocket_isAvailable(PooledSocket* this);
static inline void PooledSocket_setAvailable(PooledSocket* this, fhgfs_bool available);
static inline void PooledSocket_setExpirationCounter(PooledSocket* this, int expirationCounter);
static inline NicAddrType_t PooledSocket_getNicType(PooledSocket* this);

struct PooledSocket
{
   Socket socket;

   fhgfs_bool available; // == !acquired
   int expirationCounter; // 0 means "doesn't expire", >0 means "numRequests until expiration"
   NicAddrType_t nicType; // same as the interface for which this conn was established
};


void _PooledSocket_init(PooledSocket* this, NicAddrType_t nicType)
{
   _Socket_init( (Socket*)this);

   this->available = fhgfs_false;
   this->expirationCounter = 0;
   this->nicType = nicType;
}


PooledSocket* _PooledSocket_construct(NicAddrType_t nicType)
{
   PooledSocket* this = (PooledSocket*)os_kmalloc(sizeof(*this) );

   _PooledSocket_init(this, nicType);

   return this;
}

void _PooledSocket_uninit(Socket* this)
{
   _Socket_uninit(this);
}

void _PooledSocket_destruct(Socket* this)
{
   _PooledSocket_uninit(this);

   os_kfree(this);
}

/**
 * Tests the expiration counter and decreases its value if necessary.
 *
 * @return fhgfs_true if this socket just expired
 */
fhgfs_bool PooledSocket_testAndDecExpirationCounter(PooledSocket* this)
{
   if(likely(!this->expirationCounter) )
      return fhgfs_false;

   if(this->expirationCounter == 1)
      return fhgfs_true; // we don't decrease the value here to keep the expiration stable

   this->expirationCounter--;

   return fhgfs_false;
}

fhgfs_bool PooledSocket_isAvailable(PooledSocket* this)
{
   return this->available;
}

void PooledSocket_setAvailable(PooledSocket* this, fhgfs_bool available)
{
   this->available = available;
}

void PooledSocket_setExpirationCounter(PooledSocket* this, int expirationCounter)
{
   this->expirationCounter = expirationCounter;
}

NicAddrType_t PooledSocket_getNicType(PooledSocket* this)
{
   return this->nicType;
}

#endif /*POOLEDSOCKET_H_*/
