#ifndef FHGFSOPSREMOTING_H_
#define FHGFSOPSREMOTING_H_

#include <include_closed/common/Common.h>
#include <include_closed/filesystem/FsObjectInfo.h>
#include <include_closed/filesystem/FsDirInfo.h>
#include <include_closed/filesystem/FsFileInfo.h>
#include <include_closed/common/storage/StorageDefinitions.h>
#include <include_closed/common/toolkit/MetadataTk.h>
#include <include_open/common/storage/Open_StorageErrors.h>
#include <include_closed/net/filesystem/RemotingIOInfo.h>


struct StripePattern; // forward declaration


extern FhgfsOpsErr FhgfsOpsRemoting_opendir(const OwnerMinInfo* entryInfo, FsDirInfo* outDirInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_listdirIncremental(const OwnerMinInfo* entryInfo,
   FsDirInfo* outDirInfo, const char* lastEntry, int lastStage, unsigned maxOutNames);
extern FhgfsOpsErr FhgfsOpsRemoting_listdirFromOffset(const OwnerMinInfo* entryInfo,
   FsDirInfo* outDirInfo, uint64_t offset, unsigned maxOutNames);
extern FhgfsOpsErr FhgfsOpsRemoting_stat(App* app, char* path, fhgfs_stat* fhgfsStat);
extern FhgfsOpsErr FhgfsOpsRemoting_statDirect(App* app, const OwnerMinInfo* entryInfo,
   fhgfs_stat* fhgfsStat);
extern FhgfsOpsErr FhgfsOpsRemoting_setAttr(App* app, const OwnerMinInfo* entryInfo,
   SettableFileAttribs* fhgfsAttr, int validAttribs);
extern FhgfsOpsErr FhgfsOpsRemoting_mkdir(App* app, const OwnerMinInfo* parentInfo,
   const char* entryName, unsigned userID, unsigned groupID, int mode, OwnerMinInfo* outEntryInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_rmdir(App* app, const OwnerMinInfo* parentInfo,
   const char* entryName);
extern FhgfsOpsErr FhgfsOpsRemoting_mkfile(App* app, const OwnerMinInfo* parentInfo,
   const char* entryName, unsigned userID, unsigned groupID, int mode, OwnerMinInfo* outEntryInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_unlinkfile(App* app, const OwnerMinInfo* parentInfo,
   const char* entryName);
extern FhgfsOpsErr FhgfsOpsRemoting_openfile(const OwnerMinInfo* entryInfo, RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_closefile(const OwnerMinInfo* entryInfo,
   RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsRemoting_writefile(const char __user *buf, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsRemoting_writefileThreaded(const char __user *buf, size_t size,
   loff_t offset, RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsRemoting_readfile(char __user *buf, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsRemoting_readfileThreaded(char *buf, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_rename(App* app, char* pathFrom, char* pathTo);
extern FhgfsOpsErr FhgfsOpsRemoting_truncfile(App* app, const OwnerMinInfo* entryInfo, loff_t size);
extern FhgfsOpsErr FhgfsOpsRemoting_fsyncfile(RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_statStoragePath(App* app, int64_t* outSizeTotal,
   int64_t* outSizeFree);

extern FhgfsOpsErr FhgfsOpsRemoting_validateSubentry(App* app, const OwnerMinInfo* parentOwnerInfo,
   const char* entryName, const OwnerMinInfo* entryOwnerInfo);
extern FhgfsOpsErr FhgfsOpsRemoting_lookupSubentry(App* app, const OwnerMinInfo* parentOwnerInfo,
   const char* entryName, OwnerMinInfo* outEntryOwnerInfo);


#ifdef LOG_DEBUG_MESSAGES
extern void __FhgfsOpsRemoting_logDebugIOCall(const char* logContext, size_t size, loff_t offset,
   RemotingIOInfo* ioInfo);
#else
#define __FhgfsOpsRemoting_logDebugIOCall(logContext, size, offset, ioInfo)
#endif // LOG_DEBUG_MESSAGES


#endif /*FHGFSOPSREMOTING_H_*/
