#ifndef REMOTINGIOINFO_H_
#define REMOTINGIOINFO_H_

#include <include_closed/common/Common.h>
#include <include_closed/common/storage/striping/StripePattern.h>
#include <include_closed/app/App.h>


struct RemotingIOInfo;
typedef struct RemotingIOInfo RemotingIOInfo;


// inliners
static inline void RemotingIOInfo_initOpen(App* app, unsigned accessFlags, int* pMaxUsedNodeIndex,
   RemotingIOInfo* outIOInfo);
static inline void RemotingIOInfo_freeVals(RemotingIOInfo* outIOInfo);



struct RemotingIOInfo
{
      App* app;

      const char* fileHandleID;
      struct StripePattern* pattern;
      unsigned accessFlags; // OPENFILE_ACCESS_... flags

      int* pMaxUsedNodeIndex; // in and out value; zero-based pattern index (-1 means "none")
};


/**
 * Prepares a RemotingIOInfo for Remoting_openfile().
 *
 * Note: Be careful with this. This is only useful in very special cases (e.g. stateless file IO,
 * when you called FhgfsOpsRemoting_openfile() directly). You will typically rather use the
 * FhgfsInode_referenceHandle() & co routines.
 *
 * @param accessFlags OPENFILE_ACCESS_... flags
 * @param pMaxUsedNodeIndex may be NULL for open, but in case you will use the ioInfo also for
 * IO, you will probably want to set it.
 */
void RemotingIOInfo_initOpen(App* app, unsigned accessFlags, int* pMaxUsedNodeIndex,
   RemotingIOInfo* outIOInfo)
{
   outIOInfo->app = app;
   outIOInfo->fileHandleID = NULL;
   outIOInfo->pattern = NULL;
   outIOInfo->accessFlags = accessFlags;
   outIOInfo->pMaxUsedNodeIndex = pMaxUsedNodeIndex;
}

/**
 * Note: Be careful with this. This is only useful in very special cases (e.g. stateless file IO,
 * when you called Remoting_openfile() directly). You will typically rather use the
 * FhgfsInode_referenceHandle() & co routines.
 */
void RemotingIOInfo_freeVals(RemotingIOInfo* outIOInfo)
{
   SAFE_DESTRUCT(outIOInfo->pattern, StripePattern_virtualDestruct);
   SAFE_KFREE(outIOInfo->fileHandleID);
}


#endif /* REMOTINGIOINFO_H_ */
