#ifndef OPEN_COMMON_H_
#define OPEN_COMMON_H_

#include <stdarg.h>
#include <linux/types.h>
#include <linux/stddef.h>
#include <linux/compiler.h>
#include <asm/div64.h>

#include <include_open/common/Open_FhgfsTypes.h>
#include <include_open/os/Open_OsDeps.h>


#define MIN(a, b) \
   ( ( (a) < (b) ) ? (a) : (b) )
#define MAX(a, b) \
   ( ( (a) < (b) ) ? (b) : (a) )

#define SAFE_VFREE(p) \
   do{ if(p) {os_vfree(p); (p)=NULL;} } while(0)

#define SAFE_VFREE_NOSET(p) \
   do{ if(p) os_vfree(p); } while(0)

#define SAFE_KFREE(p) \
   do{ if(p) {os_kfree(p); (p)=NULL;} } while(0)

#define SAFE_KFREE_NOSET(p) \
   do{ if(p) os_kfree(p); } while(0)

#define SAFE_DESTRUCT(p, destructor) \
   do{if(p) {destructor(p); (p)=NULL;} } while(0)

#define SAFE_DESTRUCT_NOSET(p, destructor) \
   do{if(p) destructor(p); } while(0)


// module name (used for logging)
#define FHGFS_MODULE_NAME_STR          "fhgfs"
#define FHGFS_THREAD_NAME_PREFIX_STR   "fhgfs_"

// a printk-version that adds the module name and comm name
#define printk_fhgfs(levelStr, fmtStr, ...) \
   os_printk(levelStr FHGFS_MODULE_NAME_STR ": " "%s: " fmtStr, os_getCurrentCommNameStr(), \
      ## __VA_ARGS__)

// dumps stack in case of a buggy condition
#define FHGFS_BUG_ON(condition, msgStr) \
   do { \
      if(unlikely(condition) ) { \
         printk_fhgfs(KERN_WARNING, "%d:%s: BUG: %s", __LINE__, __func__, msgStr); \
         dump_stack(); \
      } \
   } while(0)



#ifdef LOG_DEBUG_MESSAGES

#define printk_fhgfs_debug(levelStr, fmtStr, ...) \
   os_printk(levelStr FHGFS_MODULE_NAME_STR ": " "%s: " fmtStr, os_getCurrentCommNameStr(), \
      ## __VA_ARGS__)

#else

#define printk_fhgfs_debug(levelStr, fmtStr, ...)

#endif // LOG_DEBUG_MESSAGES


// this macro mutes warnings about unused variables
#define IGNORE_UNUSED_VARIABLE(a)   do{ if( ((long)a)==1) {} } while(0)

// __copy_to/from_user() are declared with warnings for ignored return values. this macro mutes the
// warnings (because we check up front whether everything is correct).
#define IGNORE_RETURN_VALUE(a)      IGNORE_UNUSED_VARIABLE(a)


/*
 * Debug definitions:
 * - LOG_DEBUG_MESSAGES: Enables logging of some extra debug messages that will not be
 *    available otherwise.
 * - DEBUG_REFCOUNT: Enables debugging of ObjectReferencer::refCount. Error messages will
 *    be logged if refCount is less than zero.
 */


#endif /* OPEN_COMMON_H_ */
