#ifndef OPEN_FHGFSTYPES_H_
#define OPEN_FHGFSTYPES_H_

#include <opentk/common/OpenTk_FhgfsTypes.h>


#define FHGFS_LLONG_MAX    ((long long)(~0ULL>>1))


// printk log levels
#define FHGFS_KERN_EMERG   "<0>" /* system is unusable */
#define FHGFS_KERN_ALERT   "<1>" /* action must be taken immediately */
#define FHGFS_KERN_CRIT    "<2>" /* critical conditions */
#define FHGFS_KERN_ERR     "<3>" /* error conditions */
#define FHGFS_KERN_WARNING "<4>" /* warning conditions */
#define FHGFS_KERN_NOTICE  "<5>" /* normal but significant condition */
#define FHGFS_KERN_INFO    "<6>" /* informational */
#define FHGFS_KERN_DEBUG   "<7>" /* debug-level messages */

// system error codes
#define FHGFS_EPERM         1    // Operation not permitted
#define FHGFS_EAGAIN       11    // Try again
#define FHGFS_EINVAL       22    // Invalid argument
#define FHGFS_EREMOTEIO   121    // Remote I/O error



typedef unsigned fhgfs_in_addr;


struct fhgfs_sockaddr_in
{
   fhgfs_in_addr addr;
   unsigned short port; // port in network byte order
};
typedef struct fhgfs_sockaddr_in fhgfs_sockaddr_in;

#define FHGFS_INADDR_ANY        ( (unsigned long int) 0x00000000) // accept any address
#define FHGFS_INADDR_BROADCAST  ( (unsigned long int) 0xffffffff) // broadcast address
#define FHGFS_INADDR_LOOPBACK   ( (unsigned long int) 0x7f000001) // 127.0.0.1 (host byte order)
#define FHGFS_INADDR_NONE       ( (unsigned long int) 0xffffffff) // error address



struct fhgfs_timeval
{
   time_t      tv_sec;  // seconds
   suseconds_t tv_usec; // microseconds
};
typedef struct fhgfs_timeval fhgfs_timeval;


struct fhgfs_stat
{
   umode_t mode;
   unsigned int nlink;
   uid_t uid;
   gid_t gid;
   loff_t size;
   fhgfs_timespec atime;
   fhgfs_timespec mtime;
   fhgfs_timespec ctime; // change time (not creation time)
};
typedef struct fhgfs_stat fhgfs_stat;



#endif /* OPEN_FHGFSTYPES_H_ */
