#include <include_open/app/config/Open_MountConfig.h>
#include <open/common/Common.h>

#include <linux/parser.h>


enum {
   /* Mount options that take string arguments */
   Opt_cfgFilename,
   Opt_sysMgmtdHost,

   /* Mount options that take integer arguments */
   Opt_logLevel,
   Opt_connMgmtdPortUDP,
   Opt_connMgmtdPortTCP,

   Opt_err
};


static match_table_t fhgfs_mount_option_tokens =
{
   { Opt_cfgFilename, "cfgFilename=%s" },
   { Opt_sysMgmtdHost, "sysMgmtdHost=%s" },

   { Opt_logLevel, "logLevel=%d" },
   { Opt_connMgmtdPortUDP, "connMgmtdPortUDP=%u" },
   { Opt_connMgmtdPortTCP, "connMgmtdPortTCP=%u" },

   { Opt_err, NULL }
};



fhgfs_bool MountConfig_parseFromRawOptions(MountConfig* this, char* mountOptions)
{
   char* currentOption;

   if(!mountOptions)
   {
      printk_fhgfs_debug(FHGFS_KERN_INFO, "Mount options = <none>\n");
      return fhgfs_true;
   }


   printk_fhgfs_debug(FHGFS_KERN_INFO, "Mount options = '%s'\n", mountOptions);

   while( (currentOption = strsep(&mountOptions, ",") ) != NULL)
   {
      substring_t args[MAX_OPT_ARGS];
      int tokenID;

      if(!*currentOption)
         continue; // skip empty string

      //printk_fhgfs_debug(FHGFS_KERN_INFO, "Parsing mount option: '%s'\n", currentOption); // debug in

      tokenID = match_token(currentOption, fhgfs_mount_option_tokens, args);

      switch(tokenID)
      {
         case Opt_cfgFilename:
         {
            SAFE_KFREE(this->cfgFilename);

            // note: string will be kalloc'ed and needs to be kfree'd later
            this->cfgFilename = match_strdup(args);

            //printk_fhgfs_debug(FHGFS_KERN_INFO, "Mount option cfgFilename: '%s'\n", this->cfgFilename);
         } break;

         case Opt_sysMgmtdHost:
         {
            SAFE_KFREE(this->sysMgmtdHost);

            // note: string will be kalloc'ed and needs to be kfree'd later
            this->sysMgmtdHost = match_strdup(args);
         } break;

         case Opt_logLevel:
         {
            if(match_int(args, &this->logLevel) )
               goto err_exit_invalid_option;

            this->logLevelDefined = fhgfs_true;

            //printk_fhgfs_debug(FHGFS_KERN_INFO, "Mount option logLevel: %d\n", this->logLevel);
         } break;

         case Opt_connMgmtdPortUDP:
         {
            if(match_int(args, &this->connMgmtdPortUDP) )
               goto err_exit_invalid_option;

            this->connMgmtdPortUDPDefined = fhgfs_true;
         } break;

         case Opt_connMgmtdPortTCP:
         {
            if(match_int(args, &this->connMgmtdPortTCP) )
               goto err_exit_invalid_option;

            this->connMgmtdPortTCPDefined = fhgfs_true;
         } break;

         default:
            goto err_exit_unknown_option;
      }
   }

   return fhgfs_true;


err_exit_unknown_option:
   printk_fhgfs(FHGFS_KERN_WARNING, "Unknown mount option: '%s'\n", currentOption);
   return fhgfs_false;

err_exit_invalid_option:
   printk_fhgfs(FHGFS_KERN_WARNING, "Invalid mount option: '%s'\n", currentOption);
   return fhgfs_false;
}



