#include <include_open/common/storage/Open_StorageErrors.h>
#include <open/common/Common.h>


#define __FHGFSOPS_ERRLIST_SIZE \
   ( (sizeof(__FHGFSOPS_ERRLIST) ) / (sizeof(struct FhgfsOpsErrListEntry) ) - 1)
   /* -1 because last elem is NULL */



// Note: This is based on the FhgfsOpsErr entries
// Note: We use EREMOTEIO as a generic error here
struct FhgfsOpsErrListEntry const __FHGFSOPS_ERRLIST[] =
{
   {"Success", 0}, // FhgfsOpsErr_SUCCESS
   {"Internal error", EREMOTEIO}, // FhgfsOpsErr_INTERNAL
   {"Interrupted system call", EINTR}, // FhgfsOpsErr_INTERRUPTED
   {"Communication error", ECOMM}, // FhgfsOpsErr_COMMUNICATION
   {"Communication timeout", ETIMEDOUT}, // FhgfsOpsErr_COMMTIMEDOUT
   {"Unknown node", EREMOTEIO}, // FhgfsOpsErr_UNKNOWNNODE
   {"Node is not owner of entry", EREMOTEIO}, // FhgfsOpsErr_NOTOWNER
   {"Entry exists already", EEXIST}, // FhgfsOpsErr_EXISTS
   {"Path does not exist", ENOENT}, // FhgfsOpsErr_PATHNOTEXISTS
   {"Entry is in use", EBUSY}, // FhgfsOpsErr_INUSE
   {"Dynamic attributes of entry are outdated", EREMOTEIO}, // FhgfsOpsErr_INUSE
   {"Cannot move directory to one of its subdirs", EINVAL}, // FhgfsOpsErr_PARENTTOSUBDIR
   {"Entry is not a directory", ENOTDIR}, // FhgfsOpsErr_NOTADIR
   {"Directory is not empty", ENOTEMPTY}, // FhgfsOpsErr_NOTEMPTY
   {"No space left", ENOSPC}, // FhgfsOpsErr_NOSPACE
   {NULL, 0}
};



/**
 * @return static human-readable error string
 */
const char* FhgfsOpsErr_toErrString(FhgfsOpsErr errCode)
{
   size_t unsignedErrCode = (size_t)errCode;

   if(likely(unsignedErrCode < __FHGFSOPS_ERRLIST_SIZE) )
      return __FHGFSOPS_ERRLIST[unsignedErrCode].errString;


   printk_fhgfs(KERN_WARNING, "Bug: Invalid errCode given to FhgfsOpsErr_toErrString(): %d/%u\n",
      (int)errCode, (unsigned)errCode);

   return "Invalid (undefined) error code";
}

/**
 * @return negative linux error code
 */
int FhgfsOpsErr_toSysErr(FhgfsOpsErr errCode)
{
   size_t unsignedErrCode = (size_t)errCode;

   if(likely(unsignedErrCode < __FHGFSOPS_ERRLIST_SIZE) )
      return -(__FHGFSOPS_ERRLIST[errCode].sysErr);


   printk_fhgfs(KERN_WARNING, "Bug: Invalid errCode given to FhgfsOpsErr_toSysErr(): %d/%u\n",
      (int)errCode, (unsigned)errCode);

   return -EPERM;
}
