#include <open/common/Common.h>
#include <include_open/common/threading/Open_Condition.h>
#include <open/common/toolkit/TimeTk.h>

#include <linux/sched.h>


struct Condition
{
   wait_queue_head_t queue;
};


void Condition_init(Condition* this)
{
   init_waitqueue_head(&this->queue);
}

Condition* Condition_construct(void)
{
   Condition* this = (Condition*)os_kmalloc(sizeof(*this) );

   Condition_init(this);

   return this;
}

void Condition_uninit(Condition* this)
{
}

void Condition_destruct(Condition* this)
{
   Condition_uninit(this);

   kfree(this);
}

cond_wait_res_t Condition_wait(Condition* this, Mutex* mutex)
{
   cond_wait_res_t retVal;
   wait_queue_t wait;

   init_waitqueue_entry(&wait, current);
   add_wait_queue(&this->queue, &wait);

   set_current_state(TASK_INTERRUPTIBLE);

   Mutex_unlock(mutex);
   schedule();
   retVal = signal_pending(current) ? COND_WAIT_SIGNAL : COND_WAIT_SUCCESS;

   __set_current_state(TASK_RUNNING);

   remove_wait_queue(&this->queue, &wait);

   Mutex_lock(mutex);

   return retVal;
}

cond_wait_res_t Condition_timedwait(Condition* this, Mutex* mutex, int timeoutMS)
{
   cond_wait_res_t retVal;
   long __timeout;

   wait_queue_t wait;
   init_waitqueue_entry(&wait, current);
   add_wait_queue(&this->queue, &wait);

   set_current_state(TASK_INTERRUPTIBLE);

   Mutex_unlock(mutex);

   // timeval to jiffies
   __timeout = TimeTk_msToJiffiesSchedulable(timeoutMS);

   // wait
   if( (__timeout = schedule_timeout(__timeout) ) )
      retVal = signal_pending(current) ? COND_WAIT_SIGNAL : COND_WAIT_SUCCESS;
   else
      retVal = COND_WAIT_TIMEOUT;

   __set_current_state(TASK_RUNNING);

   remove_wait_queue(&this->queue, &wait);

   Mutex_lock(mutex);

   return retVal;
}

void Condition_broadcast(Condition* this)
{
   wake_up_interruptible(&this->queue);
}


/**
 * Note: this routine simply delegates to broadcast()
 */
void Condition_signal(Condition* this)
{
   Condition_broadcast(this);
}
