#ifndef FHGFSOPSFILE_H_
#define FHGFSOPSFILE_H_

#include <include_closed/app/App.h>
#include <include_open/common/threading/Open_Mutex.h>
#include <include_open/common/toolkit/Open_Time.h>
#include <include_closed/net/filesystem/FhgfsOpsRemoting.h>
#include <include_closed/filesystem/FsDirInfo.h>
#include <include_closed/filesystem/FsFileInfo.h>
#include <include_closed/toolkit/NoAllocBufferStore.h>
#include <include_closed/common/toolkit/MetadataTk.h>
#include <open/common/Common.h>
#include "FhgfsOps_versions.h"
#include "FhgfsOpsInode.h"


#include <linux/fs.h>
#include <linux/vfs.h>
#include <linux/pagemap.h>
#include <linux/uio.h>


#ifndef SEEK_SET
#define SEEK_SET  0  /* seek relative to beginning of file */
#define SEEK_CUR  1  /* seek relative to current file position */
#define SEEK_END  2  /* seek relative to end of file */
#endif


// forward declaration
struct App;


extern struct file_operations fhgfs_file_ops;
extern struct file_operations fhgfs_file_pagecache_ops;

extern struct file_operations fhgfs_dir_ops;

extern struct address_space_operations fhgfs_address_ops;
extern struct address_space_operations fhgfs_address_pagecache_ops;


extern loff_t FhgfsOps_llseek(struct file *file, loff_t offset, int origin);

extern int FhgfsOps_opendirIncremental(struct inode* inode, struct file* file);
extern int FhgfsOps_readdirIncremental(struct file* file, void* buf, filldir_t filldir);
extern int FhgfsOps_releasedir(struct inode* inode, struct file* file);

extern int FhgfsOps_open(struct inode* inode, struct file* file);
extern int FhgfsOps_release(struct inode* inode, struct file* file);
extern int FhgfsOps_fsync(struct file* file, struct dentry* dentry, int datasync);

extern int FhgfsOps_readlink(struct dentry* dentry, char __user* buf, int size);


extern ssize_t FhgfsOps_read(struct file* file, char __user *buf, size_t size,
   loff_t* offsetPointer);
extern ssize_t FhgfsOps_write(struct file* file, const char __user *buf, size_t size,
   loff_t* offsetPointer);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
extern ssize_t FhgfsOps_aio_read(struct kiocb *iocb, char __user *buf, size_t count, loff_t pos);
extern ssize_t FhgfsOps_aio_write(struct kiocb *iocb, const char __user *buf, size_t count,
   loff_t pos);
#else
extern ssize_t FhgfsOps_aio_read(struct kiocb *iocb, const struct iovec *iov,
   unsigned long nr_segs, loff_t pos);
extern ssize_t FhgfsOps_aio_write(struct kiocb *iocb, const struct iovec *iov,
   unsigned long nr_segs, loff_t pos);
#endif // LINUX_VERSION_CODE
extern int FhgfsOps_mmap(struct file *, struct vm_area_struct *);

extern int FhgfsOps_readpage(struct file *file, struct page *page);
extern int FhgfsOps_readpages(struct file* file, struct address_space* mapping,
   struct list_head* page_list, unsigned num_pages);
extern void __FhgfsOps_copyToCachePages(struct address_space* mapping, struct list_head* page_list,
   char* srcBuf, int bufLen, struct pagevec* plru_pvec);
extern int FhgfsOps_writepage(struct page *page, struct writeback_control *wbc);
extern int FhgfsOps_writepages(struct address_space* mapping, struct writeback_control* wbc);

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
   extern int FhgfsOps_prepare_write(struct file *file, struct page *page, unsigned from,
      unsigned to);
   extern int FhgfsOps_commit_write(struct file *file, struct page *page, unsigned from,
      unsigned to);
#else
   extern int FhgfsOps_write_begin(struct file* file, struct address_space* mapping,
      loff_t pos, unsigned len, unsigned flags, struct page** pagep, void** fsdata);
   extern int FhgfsOps_write_end(struct file* file, struct address_space* mapping,
      loff_t pos, unsigned len, unsigned copied, struct page* page, void* fsdata);
#endif // LINUX_VERSION_CODE

extern sector_t FhgfsOps_bmap(struct address_space *, sector_t);
extern ssize_t FhgfsOps_directIO(int rw, struct kiocb *iocb, const struct iovec *iov, loff_t pos,
   unsigned long nr_segs);

extern int __FhgfsOps_readpage_sync(struct file* file, struct page* page, loff_t offset);
extern int __FhgfsOps_writepage_sync(struct file* file, struct page* page, unsigned pageOffset,
   unsigned count);
extern int __FhgfsOps_writepage_async(struct page* page, unsigned from, unsigned to);
extern ssize_t __FhgfsOps_writeStateless(struct inode* inode, const char __user *buf, size_t size,
   loff_t offset);


// getters & setters
static inline FsObjectInfo* __FhgfsOps_getObjectInfo(struct file* file);
static inline void __FhgfsOps_setObjectInfo(FsObjectInfo* objectInfo, struct file* file);
static inline FsDirInfo* __FhgfsOps_getDirInfo(struct file* file);
static inline void __FhgfsOps_setDirInfo(FsDirInfo* dirInfo, struct file* outFile);
static inline FsFileInfo* __FhgfsOps_getFileInfo(struct file* file);
static inline void __FhgfsOps_setFileInfo(FsFileInfo* fileInfo, struct file* outFile);



FsObjectInfo* __FhgfsOps_getObjectInfo(struct file* file)
{
   return (FsObjectInfo*)file->private_data;
}

void __FhgfsOps_setObjectInfo(FsObjectInfo* objectInfo, struct file* file)
{
   file->private_data = objectInfo;
}

FsDirInfo* __FhgfsOps_getDirInfo(struct file* file)
{
   return (FsDirInfo*)file->private_data;
}

void __FhgfsOps_setDirInfo(FsDirInfo* dirInfo, struct file* outFile)
{
   outFile->private_data = dirInfo;
}

FsFileInfo* __FhgfsOps_getFileInfo(struct file* file)
{
   return (FsFileInfo*)file->private_data;
}

void __FhgfsOps_setFileInfo(FsFileInfo* fileInfo, struct file* outFile)
{
   outFile->private_data = fileInfo;
}


#endif /*FHGFSOPSFILE_H_*/
