#ifndef FHGFSOPSINODE_H_
#define FHGFSOPSINODE_H_

#include <include_closed/app/App.h>
#include <include_open/common/threading/Open_Mutex.h>
#include <include_open/common/toolkit/Open_Time.h>
#include <include_closed/net/filesystem/FhgfsOpsRemoting.h>
#include <include_closed/filesystem/FsDirInfo.h>
#include <include_closed/filesystem/FsFileInfo.h>
#include <include_closed/toolkit/NoAllocBufferStore.h>
#include <open/common/Common.h>
#include "FhgfsInode.h"
#include "FhgfsOps_versions.h"

#include <linux/fs.h>
#include <linux/vfs.h>


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
   #define currentFsUserID    (current->fsuid)
   #define currentFsGroupID   (current->fsgid)
#else
   #define currentFsUserID       current_fsuid()
   #define currentFsGroupID      current_fsgid()
#endif


// forward declaration
struct App;


extern struct dentry* FhgfsOps_lookup(struct inode* parentDir, struct dentry* dentry,
   struct nameidata* nameidata);

extern int FhgfsOps_getattr(struct vfsmount* mnt, struct dentry* dentry, struct kstat* kstat);
extern int FhgfsOps_setattr(struct dentry* dentry, struct iattr* iattr);

extern int FhgfsOps_mkdir(struct inode* dir, struct dentry* dentry, int mode);
extern int FhgfsOps_rmdir(struct inode* dir, struct dentry* dentry);

extern int FhgfsOps_create(struct inode* dir, struct dentry* dentry, int mode,
   struct nameidata* nameidata);
extern int FhgfsOps_unlink(struct inode* dir, struct dentry* dentry);

extern int FhgfsOps_symlink(struct inode* dir, struct dentry* dentry, const char* to);

extern int FhgfsOps_link(struct dentry* dentryFrom, struct inode* inode, struct dentry* dentryTo);

extern void* FhgfsOps_follow_link(struct dentry* dentry, struct nameidata* nd);
extern void FhgfsOps_put_link(struct dentry *dentry, struct nameidata* nd, void* p);

extern int FhgfsOps_rename(struct inode* inodeDirFrom, struct dentry* dentryFrom,
   struct inode* inodeDirTo, struct dentry* dentryTo);

extern void FhgfsOps_truncate(struct inode* inode);
extern int FhgfsOps_vmtruncate(struct inode* inode, loff_t offset);


extern fhgfs_bool FhgfsOps_initInodeCache(void);
extern void FhgfsOps_destroyInodeCache(void);
extern struct inode* FhgfsOps_alloc_inode(struct super_block* sb);
extern void FhgfsOps_destroy_inode(struct inode* inode);
extern struct inode* __FhgfsOps_newInode(struct super_block* sb, struct kstat* kstat, dev_t dev,
   OwnerMinInfo* entryInfo);
extern int __FhgfsOps_instantiateInode(struct dentry* dentry, OwnerMinInfo* entryInfo);
extern int __FhgfsOps_refreshInode(App* app, struct inode* inode);
extern int __FhgfsOps_revalidateDirInode(App* app, struct inode* inode);
extern int __FhgfsOps_revalidateMapping(App* app, struct inode* inode);


// inliners
static inline void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, struct inode* outInode);
static inline void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat,
   struct inode* outInode);
static inline void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode);
static inline void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat, struct inode* outInode);


/**
 * Note: acquires i_lock spinlock to protect i_size and i_blocks
 */
void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   spin_lock(&outInode->i_lock); // I _ L O C K

   __FhgfsOps_applyStatSizeToInode(kstat, outInode);

   spin_unlock(&outInode->i_lock); // I _ U N L O C K
}

/**
 * Note: Caller must hold i_lock.
 */
void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat, struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   __FhgfsOps_applyStatSizeToInode(kstat, outInode);
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 */
void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode)
{
   // remote attribs (received from nodes)
   outInode->i_mode = kstat->mode;
   outInode->i_uid = kstat->uid;
   outInode->i_gid = kstat->gid;
   outInode->i_atime = kstat->atime;
   outInode->i_mtime = kstat->mtime;
   outInode->i_ctime = kstat->ctime;
   outInode->i_nlink = kstat->nlink;

   outInode->i_blkbits = FHGFS_INODE_BLOCKBITS;

#ifdef KERNEL_HAS_IBLKSIZE
   outInode->i_blksize = FHGFS_INODE_BLOCKSIZE;
#endif // KERNEL_HAS_IBLKSIZE
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 * Note: Caller must hold i_lock.
 */
void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat, struct inode* outInode)
{
   i_size_write(outInode, kstat->size);
   outInode->i_blocks = (512-1 + kstat->size) >> 9;
      // note: i_blocks is in 512-byte units (see man stat(2) )
      // note: things with a positive size have at least one block with this
}

#endif /* FHGFSOPSINODE_H_ */
