#ifndef FHGFSOPS_VERSIONS_H_
#define FHGFSOPS_VERSIONS_H_

#include <open/common/Common.h>
#include <include_closed/app/log/Logger.h>
#include <include_closed/app/App.h>
#include <include_closed/net/filesystem/FhgfsOpsRemoting.h>
#include <include_closed/filesystem/FsDirInfo.h>
#include <include_closed/filesystem/FsFileInfo.h>

#include <linux/fs.h>
#include <linux/vfs.h>
#include <linux/pagevec.h>
#include <linux/pagemap.h>
#include <linux/page-flags.h>


#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,17)
extern struct super_block* __FhgfsOps_getSB(struct file_system_type *fs_type,
   int flags, const char *dev_name, void *data);
#else
extern int __FhgfsOps_getSB(struct file_system_type *fs_type,
   int flags, const char *dev_name, void *data, struct vfsmount *mnt);
#endif // LINUX_VERSION_CODE


#ifndef KERNEL_HAS_STATFS_DENTRY
extern int FhgfsOps_statfs(struct super_block* sb, struct kstatfs* kstatfs);
#else
extern int FhgfsOps_statfs(struct dentry* dentry, struct kstatfs* kstatfs);
#endif // KERNEL_HAS_STATFS_DENTRY


#ifndef KERNEL_HAS_FLUSH_OWNER
extern int FhgfsOps_flush(struct file *file);
#else
extern int FhgfsOps_flush(struct file *file, fl_owner_t id);
#endif // KERNEL_HAS_FLUSH_OWNER

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,23)
extern void FhgfsOps_initInodeOnce(void* inode, struct kmem_cache* cache, unsigned long flags);
#elif LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26)
extern void FhgfsOps_initInodeOnce(struct kmem_cache* cache, void* inode);
#else
extern void FhgfsOps_initInodeOnce(void* inode);
#endif // LINUX_VERSION_CODE


////////////// start of kernel method emulators //////////////


#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26)
extern loff_t generic_file_llseek_unlocked(struct file *file, loff_t offset, int origin);
#endif // LINUX_VERSION_CODE

#ifndef KERNEL_HAS_DROP_NLINK
extern void drop_nlink(struct inode *inode);
#endif // KERNEL_HAS_DROP_NLINK

#ifndef KERNEL_HAS_CLEAR_NLINK
extern void clear_nlink(struct inode *inode);
#endif // KERNEL_HAS_CLEAR_NLINK

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,27)
static inline void pagevec_lru_add_file(struct pagevec *pvec);
static inline void __pagevec_lru_add_file(struct pagevec *pvec);
#endif // LINUX_VERSION_CODE

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26)
static inline int trylock_page(struct page *page);
#endif // LINUX_VERSION_CODE

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,21)
static inline void mapping_set_error(struct address_space *mapping, int error);
#endif // LINUX_VERSION_CODE


#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,27)

/**
 * Note: This is just a wrapper that calls the old function name for old kernels.
 */
void pagevec_lru_add_file(struct pagevec *pvec)
{
   pagevec_lru_add(pvec);
}

/**
 * Note: This is just a wrapper that calls the old function name for old kernels.
 */
void __pagevec_lru_add_file(struct pagevec *pvec)
{
   __pagevec_lru_add(pvec);
}

#endif // LINUX_VERSION_CODE


#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26)

/**
 * Note: This is just an emulator that does the job for old kernels.
 */
int trylock_page(struct page *page)
{
   return (likely(!test_and_set_bit(PG_locked, &page->flags) ) );
}

#endif // LINUX_VERSION_CODE

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,21)

/**
 * Note: This is just an emulator that does the job for old kernels.
 */
void mapping_set_error(struct address_space *mapping, int error)
{
   if(unlikely(error) )
   {
      if(error == -ENOSPC)
         set_bit(AS_ENOSPC, &mapping->flags);
      else
         set_bit(AS_EIO, &mapping->flags);
   }
}

#endif // LINUX_VERSION_CODE


#endif /*FHGFSOPS_VERSIONS_H_*/
