#include <open/common/Common.h>
#include <include_open/os/Open_OsDeps.h>
#include <include_open/common/Open_FhgfsTypes.h>
#include <include_closed/common/net/sock/NicAddress.h>
#include <include_closed/common/net/sock/Socket.h>
#include <include_open/toolkit/Open_SignalTk.h>

#include <linux/netdevice.h>
#include <linux/in.h>
#include <linux/inetdevice.h>


fhgfs_bool os_checkCompileTimeAssumptions()
{
   if(strcmp(KERN_WARNING, FHGFS_KERN_WARNING) )
   {
      printk_fhgfs(KERN_WARNING, "printk log level definition unexpected: "
         "assumed: %s; actual: %s\n",
         FHGFS_KERN_WARNING, KERN_WARNING);
      return fhgfs_false;
   }

   if(FHGFS_IFNAMSIZ < IFNAMSIZ)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "IFNAMSIZ: assumed: %d; actual: %d\n",
         (int)FHGFS_IFNAMSIZ, (int)IFNAMSIZ);
      return fhgfs_false;
   }

   if(FHGFS_IFHWADDRLEN < IFHWADDRLEN)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "IFHWADDRLEN: assumed: %d; actual: %d\n",
         (int)FHGFS_IFHWADDRLEN, (int)IFHWADDRLEN);
      return fhgfs_false;
   }

   if(sizeof(fhgfs_sigset_t) < sizeof(sigset_t) )
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "sizeof(sigset_t): assumed: %d; actual: %d\n",
         (int)sizeof(fhgfs_sigset_t), (int)sizeof(sigset_t) );
      return fhgfs_false;
   }

   if(!__os_checkCompileTimeErrDefs() )
      return fhgfs_false;
   
   if(!SocketTk_checkCompileTimeAssumptions() )
      return fhgfs_false;

   return fhgfs_true;
}

fhgfs_bool __os_checkCompileTimeErrDefs(void)
{
   // TODO: make these defs external constants to get rid of the checking

   if(FHGFS_EPERM != EPERM)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Value of EPERM: assumed: %d; actual: %d\n",
         (int)FHGFS_EPERM, (int)EPERM);
      return fhgfs_false;
   }

   if(FHGFS_EAGAIN != EAGAIN)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Value of EAGAIN: assumed: %d; actual: %d\n",
         (int)FHGFS_EAGAIN, (int)EAGAIN);
      return fhgfs_false;
   }

   if(FHGFS_EINVAL != EINVAL)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Value of EINVAL: assumed: %d; actual: %d\n",
         (int)FHGFS_EINVAL, (int)EINVAL);
      return fhgfs_false;
   }

   if(FHGFS_EREMOTEIO != EREMOTEIO)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Value of EREMOTEIO: assumed: %d; actual: %d\n",
         (int)FHGFS_EREMOTEIO, (int)EREMOTEIO);
      return fhgfs_false;
   }

   return fhgfs_true;
}

void* os_kmalloc(size_t size)
{
   return kmalloc(size, GFP_NOFS);
}

void* os_kzalloc(size_t size)
{
   return kzalloc(size, GFP_NOFS);
}

void os_kfree(const void* addr)
{
   kfree(addr);
}

void* os_vmalloc(unsigned long size)
{
   return vmalloc(size);
}

void os_vfree(void* addr)
{
   vfree(addr);
}

unsigned long __os_get_free_page()
{
   return __get_free_page(GFP_NOFS);
}

void os_free_page(unsigned long addr)
{
   free_page(addr);
}

void* os_memcpy(void* to, const void* from, size_t n)
{
   return memcpy(to, from, n);
}

void* os_memset(void* s, int c, size_t n)
{
   return memset(s, c, n);
}

int os_memcmp(const void* cs, const void* ct, size_t count)
{
   return memcmp(cs, ct, count);
}

int __os_copy_from_user(void *dst, const void __user *src, unsigned size)
{
   return __copy_from_user(dst, src, size);
}

int __os_copy_to_user(void __user *dst, const void *src, unsigned size)
{
   return __copy_to_user(dst, src, size);
}

int os_printk(const char* fmt, ...)
{
   int retVal;
   va_list ap;

   va_start(ap, fmt);

   retVal = vprintk(fmt, ap);

   va_end(ap);

   return retVal;
}

int os_sprintf(char* buf, const char* fmt, ...)
{
   int retVal;
   va_list ap;

   va_start(ap, fmt);

   retVal = vsprintf(buf, fmt, ap);

   va_end(ap);

   return retVal;
}

/**
 * @return number of bytes that would have been written, even if buf size was too small
 * (typically, you would prefer scnprintf() ) 
 */
int os_snprintf(char* buf, size_t size, const char* fmt, ...)
{
   int retVal;
   va_list ap;

   va_start(ap, fmt);

   retVal = vsnprintf(buf, size, fmt, ap);

   va_end(ap);

   return retVal;
}

int os_vsnprintf(char* buf, size_t size, const char* fmt, va_list args)
{
   return vsnprintf(buf, size, fmt, args);
}

/**
 * @return number of bytes that actually have been written (always <= size)
 */
int os_scnprintf(char* buf, size_t size, const char* fmt, ...)
{
   int retVal;
   va_list ap;

   va_start(ap, fmt);

   retVal = vscnprintf(buf, size, fmt, ap);

   va_end(ap);

   return retVal;
}

size_t os_strlen(const char* s)
{
   return strlen(s);
}

int os_strcmp(const char* s1, const char* s2)
{
   return strcmp(s1, s2);
}

int os_strnicmp(const char *s1, const char *s2, size_t n)
{
   return strnicmp(s1, s2, n);
}

char* os_strcpy(char* dest, const char* src)
{
   return strcpy(dest, src);
}

char* os_strchr(const char* s, int c)
{
   return strchr(s, c);
}

char* os_strrchr(const char* s, int c)
{
   return strrchr(s, c);
}

const char* os_getCurrentCommNameStr()
{
   return current->comm;
}

pid_t os_getCurrentPID()
{
   return current->pid;
}
