#ifndef OSTYPECONVERSION_INTERNAL_H_
#define OSTYPECONVERSION_INTERNAL_H_

#include <open/common/Common.h>
#include <include_open/os/Open_OsTypeConversion.h>
#include <include_closed/common/storage/StorageDefinitions.h>

#include <linux/fs.h>


static inline int OsTypeConv_openFlagsOsToFhgfs(int osFlags);
static inline void OsTypeConv_kstatFhgfsToOs(fhgfs_stat* fhgfsStat, struct kstat* kStat);
static inline void OsTypeConv_iattrOsToFhgfs(struct iattr* iAttr, SettableFileAttribs* fhgfsAttr,
   int* outValidAttribs);



/**
 * @return OPENFILE_ACCESS_... flags
 */
int OsTypeConv_openFlagsOsToFhgfs(int osFlags)
{
   int fhgfsFlags = 0;

   if(osFlags & O_RDWR)
   {
      fhgfsFlags |= OPENFILE_ACCESS_READWRITE;
   }
   else
   if(osFlags & O_WRONLY)
   {
      fhgfsFlags |= OPENFILE_ACCESS_WRITE;
   }
   else
   {
      fhgfsFlags |= OPENFILE_ACCESS_READ;
   }


   if(osFlags & O_APPEND)
   {
      fhgfsFlags |= OPENFILE_ACCESS_APPEND;
   }

   if(osFlags & O_TRUNC)
   {
      fhgfsFlags |= OPENFILE_ACCESS_TRUNC;
   }

   return fhgfsFlags;
}

/**
 * @param kStat unused fields will be set to zero
 */
void OsTypeConv_kstatFhgfsToOs(fhgfs_stat* fhgfsStat, struct kstat* kStat)
{
   os_memset(kStat, 0, sizeof(*kStat) );

   kStat->mode = fhgfsStat->mode;
   kStat->nlink = fhgfsStat->nlink;
   kStat->uid = fhgfsStat->uid;
   kStat->gid = fhgfsStat->gid;
   kStat->size = fhgfsStat->size;
   kStat->atime.tv_sec = fhgfsStat->atime.tv_sec;
   kStat->atime.tv_nsec = fhgfsStat->atime.tv_nsec;
   kStat->mtime.tv_sec = fhgfsStat->mtime.tv_sec;
   kStat->mtime.tv_nsec = fhgfsStat->mtime.tv_nsec;
   kStat->ctime.tv_sec = fhgfsStat->ctime.tv_sec; // change time (not creation time)
   kStat->ctime.tv_nsec = fhgfsStat->ctime.tv_nsec; // change time (not creation time)
}

void OsTypeConv_iattrOsToFhgfs(struct iattr* iAttr, SettableFileAttribs* fhgfsAttr,
   int* outValidAttribs)
{
   *outValidAttribs = 0;

   if(iAttr->ia_valid & ATTR_MODE)
   {
      (*outValidAttribs) |= SETATTR_CHANGE_MODE;
      fhgfsAttr->mode = iAttr->ia_mode;
   }

   if(iAttr->ia_valid & ATTR_UID)
   {
      (*outValidAttribs) |= SETATTR_CHANGE_USERID;
      fhgfsAttr->userID = iAttr->ia_uid;
   }

   if(iAttr->ia_valid & ATTR_GID)
   {
      (*outValidAttribs) |= SETATTR_CHANGE_GROUPID;
      fhgfsAttr->groupID = iAttr->ia_gid;
   }

   if(iAttr->ia_valid & ATTR_MTIME)
   {
      (*outValidAttribs) |= SETATTR_CHANGE_MODIFICATIONTIME;
      fhgfsAttr->modificationTimeSecs = iAttr->ia_mtime.tv_sec;
   }

   if(iAttr->ia_valid & ATTR_ATIME)
   {
      (*outValidAttribs) |= SETATTR_CHANGE_LASTACCESSTIME;
      fhgfsAttr->lastAccessTimeSecs = iAttr->ia_atime.tv_sec;
   }
}

#endif /* OSTYPECONVERSION_INTERNAL_H_ */
