#include "Program.h"
#include <include_closed/app/App.h>
#include <include_open/common/toolkit/Open_SocketTk.h>
#include <open/filesystem/FhgfsOpsSuper.h>
#include <open/filesystem/FhgfsOpsInode.h>
#include <open/filesystem/ProcFs.h>


Program program;


int Program_main()
{
//   int appRes;
   fhgfs_bool assumptionsRes;
   fhgfs_bool sockRes;
   fhgfs_bool cacheRes;

   assumptionsRes = os_checkCompileTimeAssumptions();
   if(!assumptionsRes)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Compile-time assumptions were wrong. "
         "Contact support@fhgfs.com");
      return APPCODE_PROGRAM_ERROR;
   }

   sockRes = SocketTk_initOnce();
   if(!sockRes)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "SocketTk initialization failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   cacheRes = FhgfsOps_initInodeCache();
   if(!cacheRes)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "Inode cache initialization failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   program.registerRes = FhgfsOps_registerFilesystem();
   if(program.registerRes)
   {
      printk_fhgfs(FHGFS_KERN_WARNING, "File system registration failed\n");
      return APPCODE_PROGRAM_ERROR;
   }

   printk_fhgfs(FHGFS_KERN_INFO, "File system registered. Type: fhgfs. Version: %s\n",
      FHGFS_VERSION);

   ProcFs_createGeneralDir();

   return 0;
}

void Program_exit()
{
   ProcFs_removeGeneralDir();

   if(!program.registerRes)
   {
      int unregisterRes;

      unregisterRes = FhgfsOps_unregisterFilesystem();
      if(unregisterRes)
         printk_fhgfs(FHGFS_KERN_WARNING, "File system deregistration failed\n");
   }

   FhgfsOps_destroyInodeCache();

   SocketTk_uninitOnce();

   printk_fhgfs(FHGFS_KERN_INFO, "fhgfs_client unloaded.\n");
}
