#ifndef MUTEX_H_
#define MUTEX_H_

#include <common/Common.h>


#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif // LINUX_VERSION_CODE


struct Mutex;
typedef struct Mutex Mutex;

static inline void Mutex_init(Mutex* this);
static inline Mutex* Mutex_construct(void);
static inline void Mutex_uninit(Mutex* this);
static inline void Mutex_destruct(Mutex* this);

static inline void Mutex_lock(Mutex* this);
static inline void Mutex_unlock(Mutex* this);

struct Mutex
{
   struct semaphore sema;
};

void Mutex_init(Mutex* this)
{
   init_MUTEX(&this->sema);
}

Mutex* Mutex_construct(void)
{
   Mutex* this = (Mutex*)kmalloc(sizeof(Mutex), GFP_KERNEL);
   
   Mutex_init(this);
   
   return this;
}

void Mutex_uninit(Mutex* this)
{
}

void Mutex_destruct(Mutex* this)
{
   Mutex_uninit(this);
   
   kfree(this);
}

void Mutex_lock(Mutex* this)
{
   down(&this->sema);
}

void Mutex_unlock(Mutex* this)
{
   up(&this->sema);
}


#endif /*MUTEX_H_*/
